#include <zombie_escape>

// Definitions
#define SOUND_MAX_LENGTH 32

// Setting file
new const ZE_SETTING_RESOURCES[] = "zombie_escape.ini"

// Default access
new g_szAccessFlags[] = "d"

// Default sounds
new const szSwarmSound[][] = {
	"ambience/the_horror2.wav"
}

// Cvars Variables
new g_pCvarChance,
		g_pCvarMinPlayers,
		g_pCvarNotice,
		g_pCvarSound,
		g_pCvarDamage

// Global Variables
new g_iMaxPlayers,
		bool:g_bIsSwarm,
		bool:g_bSwarmNext

// Dynamic Arrays
new Array:g_szSwarmSound

// Forward called after server activation.
public plugin_init() {
	// Load plugin
	register_plugin("[ZE] Gamemode: Swarm", "1.2", "z0h1r-LK")
	
	// Hook Chain
	RegisterHookChain(RG_CBasePlayer_TakeDamage, "PlayerTakeDamage_Post", 1)
	
	// Cvars
	g_pCvarChance = register_cvar("ze_swarm_chance", "20")
	g_pCvarMinPlayers = register_cvar("ze_swarm_minplayers", "4")
	g_pCvarNotice = register_cvar("ze_swarm_notice", "1")
	g_pCvarSound = register_cvar("ze_swarm_sound", "1")
	g_pCvarDamage = register_cvar("ze_swarm_damage", "4")
	
	// Console command(s)
	register_concmd("ze_swarm_next", "clcmd_SwarmNext")
	
	// Default Values
	g_iMaxPlayers = get_member_game(m_nMaxPlayers)
}

// Forward allows precaching game files (models, sounds, generic files).
public plugin_precache() {
	// Initialize sounds.
	g_szSwarmSound = ArrayCreate(SOUND_MAX_LENGTH, 1)
	
	// Load sounds from external file.
	amx_load_setting_string_arr(ZE_SETTING_RESOURCES, "Sounds", "Swarm mode", g_szSwarmSound)
	
	// Save sounds in external file.
	new iIndex
	if (ArraySize(g_szSwarmSound) == 0) {
		for (iIndex = 0; iIndex < sizeof szSwarmSound; iIndex++)
			ArrayPushString(g_szSwarmSound, szSwarmSound[iIndex])
			
		// Save in external file
		amx_save_setting_string_arr(ZE_SETTING_RESOURCES, "Sounds", "Swarm mode", g_szSwarmSound)
	}
	
	// Precache sounds
	new szSound[SOUND_MAX_LENGTH]
	for (iIndex = 0; iIndex < ArraySize(g_szSwarmSound); iIndex++) {
		// Get string from dynamic array.
		ArrayGetString(g_szSwarmSound, iIndex, szSound, charsmax(szSound))
		
		if (equali(szSound[strlen(szSound)-4], ".mp3")) // .mp3
			precache_generic(szSound)
		else // .wav
			precache_sound(szSound)
	}
	
	// Load access flags from external file.
	if (!amx_load_setting_string(ZE_SETTING_RESOURCES, "Access Flags", "Swarm mode", g_szAccessFlags, charsmax(g_szAccessFlags)))
		amx_save_setting_string(ZE_SETTING_RESOURCES, "Access Flags", "Swarm mode", g_szAccessFlags)
}

// Forward allows registering native.
public plugin_natives() {
	register_native("ze_is_swarm_round", "_native_ze_is_swarm_round", 1)
	register_native("ze_swarm_set_next", "_native_ze_swarm_set_next", 1);
}

public clcmd_SwarmNext(iPlayer) {
	// Doesn't have access.
	if (!(get_user_flags(iPlayer) & read_flags(g_szAccessFlags))) {
		console_print(iPlayer, "%L", LANG_PLAYER, "CMD_NOT_ACCESS")
		return;
	}
	
	static szName[32]
	
	// Someone is already make swarm mode is next.
	if (g_bSwarmNext) {
		ze_colored_print(iPlayer, "%L", LANG_PLAYER, "CHAT_ALREADY_SWARM", szName)
		return;		
	}
	
	// Get name of admin.
	get_user_name(iPlayer, szName, charsmax(szName))
	
	// Make next round is swarm mode.
	g_bSwarmNext = true;
	ze_colored_print(0, "%L", LANG_PLAYER, "CHAT_SWARM_TURNED", szName)
}

// Forward called when every new round.
public ze_game_started() {
	// Admin is make next round is swarm mode.
	if (g_bSwarmNext) {
		g_bSwarmNext = false;
		goto turn_round;
	}
	
	// Random chance.
	if (random_num(1, get_pcvar_num(g_pCvarChance)) != 1)
		return;
		
	// Player is less from require plr.
	if (GetAllAlivePlayersNum() < get_pcvar_num(g_pCvarMinPlayers))
		return;
		
	// Turn gamemode swarm.	
	turn_round:
	g_bIsSwarm = true;
	ze_colored_print(0, "%L", LANG_PLAYER, "CHAT_SWARM_MODE")
}

// Forward called when zombie appear.
public ze_zombie_appear() {
	// This round is gamemode swarm.
	if (g_bIsSwarm) {
		// Show HUD notice turning swarm mode.
		Show_HUD()
		
		// Play sound turning swarm mode.
		new szSound[SOUND_MAX_LENGTH], iPlayer
		for (iPlayer = 1; iPlayer < g_iMaxPlayers; iPlayer++) {
			// Invalid player.
			if (!is_user_connected(iPlayer))
				continue;
				
			// Play sound when turn game is disable
			if (!get_pcvar_num(g_pCvarSound))
				break;
				
			// Get string from dynamic array.
			ArrayGetString(g_szSwarmSound, random_num(0, ArraySize(g_szSwarmSound) - 1), szSound, charsmax(szSound))
			
			// Play sound.
			PlaySound(iPlayer, szSound)
		}
	}
}

// Forward called after player infect
public ze_user_infected_pre(iVictim, iInfector, iDamage) {
	// Round is not a swarm mode.
	if (!g_bIsSwarm)
		return PLUGIN_CONTINUE;
	
	// Infector is a server.
	if (!iInfector)
		return PLUGIN_CONTINUE;
		
	// Block infect, Damage player.
	rg_multidmg_clear()
	rg_multidmg_add(iInfector, iVictim, float(iDamage) * get_pcvar_float(g_pCvarDamage), DMG_GENERIC)
	rg_multidmg_apply(iInfector, iInfector)
	return PLUGIN_HANDLED;
}

// Hook called after player take damage.
public PlayerTakeDamage_Post(iVictim, iInflector, iAttacker, Float:flDamage, bitsDamageType) {
	// Invalid player.
	if (!is_user_connected(iVictim) || !is_user_connected(iAttacker))
		return;
		
	// Victim is a zombie.
	if (ze_is_user_zombie(iVictim))
		return;
	
	// Round is not a swarm mode.
	if (!g_bIsSwarm)
		return;
		
	// Remove pain shock.
	set_member(iVictim, m_flVelocityModifier, 1.0);
}

public Show_HUD() {
	// Show HUD
	switch (get_pcvar_num(g_pCvarNotice)) {
		case 0: return; // Nothing.
		case 1: { // Normal text (yellow text).
			client_print(0, print_center, "%L", LANG_PLAYER, "NOTICE_SWARM");
		}
		case 2: { // HUD.
			set_hudmessage(0, 200, 0, -1.0, 0.3, 1, 5.0, 5.0, 0.0, 0.0);
			show_hudmessage(0, "%L", LANG_PLAYER, "NOTICE_SWARM");
		}
		case 3: { // Director-HUD.
			set_dhudmessage(0, 200, 0, -1.0, 0.3, 1, 5.0, 5.0, 0.0, 0.0);
			show_dhudmessage(0, "%L", LANG_PLAYER, "NOTICE_SWARM");			
		}
		default: { // If cvar great from 3, Set cvar 2 (HUD).
			set_pcvar_num(g_pCvarNotice, 2);
		}
	}
}

// Forward called when round end.
public ze_roundend(iWinTeam) {
	// Swarm mode is ended.
	g_bIsSwarm = false;
}

// ...
public _native_ze_is_swarm_round() {
	return g_bIsSwarm;
}

public _native_ze_swarm_set_next(bool:bSet) {
	if (g_bSwarmNext) {
		g_bSwarmNext = true
	} else {
		g_bIsSwarm = false;
	}
}